#!/usr/bin/env python

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Program: HY-menu
# Written by Antti Luukkonen <antti.luukkonen@helsinki.fi>
# 4.5.2015

import os
import re
import gtk
import gio
import signal
import subprocess
import appindicator
import urllib
import commands
import subprocess
import pwd

VERSION = '0.4'
username = pwd.getpwuid(os.getuid()).pw_name

class IndicatorPlaces:
    BOOKMARKS_PATH = '/etc/cubbli/hy-menu/hy-menu-items'

    def __init__(self):
        self.ind = appindicator.Indicator("places", "/etc/cubbli/hy-menu/hy-menu.png", appindicator.CATEGORY_APPLICATION_STATUS)
        self.ind.set_status(appindicator.STATUS_ACTIVE)
        self.fspath = self.get_fspath()
        self.update_menu()
        return

    def create_menu_item(self, label, icon_name):
        image = gtk.Image()
        image.set_from_icon_name(icon_name, 24)

        item = gtk.ImageMenuItem()
        item.set_label(label)
        item.set_image(image)
        item.set_always_show_image(True)
        return item
    
    # Icons for different menu items
    def get_bookmark_icon(self, path):
        if path.endswith("helpdesk"):
            icon_name = "stock_person-panel"
        elif path.endswith("Default.aspx"):
            icon_name = "mail_new"
        elif path.startswith("http://wpr"):
            icon_name = "stock_print"    
        else:
            f = gio.File(path)
            try:   
                info = f.query_info(gio.FILE_ATTRIBUTE_STANDARD_ICON)
                icon = info.get_icon()
                icon_name = icon.get_names()[0] if icon.get_names()[0] != '(null)' else 'folder'
            except (gio.Error, NameError):
                icon_name = "folder"
        
        return icon_name

    # This finds out which NFS-server to use
    def get_fspath(self):

        if re.match('[a-e]', username):
            home = '1'
        elif re.match('[f-j]', username):
            home = '2'
        elif re.match('[k-l]', username):
            home = '3'
        elif re.match('[m-o]', username):
            home = '4'
        elif re.match('[p-s]', username):
            home = '5'
        elif re.match('[t-z]', username):
            home = '6'
        else:
            assert False,"Invalid username!"

        if os.system("laptop-detect -v"):
            zpath = "/home/ad/fshome" + home + "/u" + home + "/"
        else:
            zpath = "smb://cella.ad.helsinki.fi/u" + home + "/"
        return zpath

    # This creates the menu
    def update_menu(self, widget = None, data = None):
        try:
            bookmarks = open(self.BOOKMARKS_PATH).readlines()
        except IOError:
            bookmarks = []        

        # Create menu
        menu = gtk.Menu()
        self.ind.set_menu(menu)

        # Add bookmarks to the menu from the /etc/cubbli/hy-menu-items
        for bm in bookmarks:
            path, label = bm.strip().partition(' ')[::2]

            if not label:
                label = os.path.basename(os.path.normpath(path))

            label = urllib.unquote(label)
            item = self.create_menu_item(label, self.get_bookmark_icon(path))
            item.connect("activate", self.on_bookmark_click, path)

            # Append the item to menu
            menu.append(item)

        # Show separator
        item = gtk.SeparatorMenuItem()
        menu.append(item)

        # HY install printers
        item = self.create_menu_item("HY Install Printers", "gnome-dev-printer-new")
        item.connect("activate", self.InstallPrinter)
        menu.append(item)

        # Show separator
        item = gtk.SeparatorMenuItem()
        menu.append(item)

        # User info
        item = self.create_menu_item("User Info", "info")
        item.connect("activate", self.userInfo)
        menu.append(item)

        # Computer info
        item = self.create_menu_item("Computer Info", "info")
        item.connect("activate", self.computerInfo)
        menu.append(item)

        # Show separator
        item = gtk.SeparatorMenuItem()
        menu.append(item)

        # Home folder menu item
        item = self.create_menu_item("Home Folder", "user-home") 
        item.connect("activate", self.on_bookmark_click, os.getenv('HOME'))
        menu.append(item)

        # This creates Z-drive menu item
        item = self.create_menu_item("Z-drive", "network")
        item.connect("activate", self.on_bookmark_click, self.fspath + username[:1] + "/" + username)
        menu.append(item)

        # Show the menu
        menu.show_all()
        return

    # Open clicked bookmark
    def on_bookmark_click(self, widget, path):
        if path.startswith("/home/ad/") or path.startswith("smb"):
            if not os.path.ismount(self.fspath):
                os.system("mount " + self.fspath)
            subprocess.Popen('/usr/bin/nautilus %s' % path, shell = True)
        else:
            subprocess.Popen('/usr/bin/xdg-open %s' % path, shell = True)
        return

    def userInfo(self, item):
        subprocess.Popen('/usr/bin/userinfo.py %s', shell = True)
        return
        
    def computerInfo(self, item):
        subprocess.Popen('/usr/bin/computerinfo.py %s', shell = True)
        return

    def InstallPrinter(self, item):
        subprocess.Popen('/usr/bin/hy-printers.py %s', shell = True)
        return

    def on_bookmarks_changed(self, filemonitor, file, other_file, event_type):
        if event_type == gio.FILE_MONITOR_EVENT_CHANGES_DONE_HINT:
            print 'Bookmarks changed, updating menu...'
            self.update_menu()
        return
           
if __name__ == "__main__":
    # Catch CTRL-C
    signal.signal(signal.SIGINT, lambda signal, frame: gtk.main_quit())

    # Run the indicator
    i = IndicatorPlaces()
    
    # Monitor bookmarks changes 
    file = gio.File(i.BOOKMARKS_PATH)
    monitor = file.monitor_file()
    monitor.connect("changed", i.on_bookmarks_changed)            
    
    # Main gtk loop
    gtk.main()

